@@ -19,7 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include "pangocairo.h"
 #include "pangocairo-private.h"
@@ -90,59 +90,62 @@ _pango_cairo_update_context (cairo_t      *cr,
   cairo_matrix_t cairo_matrix;
   cairo_surface_t *target;
   PangoMatrix pango_matrix;
-  const PangoMatrix *layout_matrix, identity_matrix = PANGO_MATRIX_INIT;
+  const PangoMatrix *current_matrix, identity_matrix = PANGO_MATRIX_INIT;
+  const cairo_font_options_t *merged_options;
+  cairo_font_options_t *old_merged_options;
   gboolean changed = FALSE;
 
   info = get_context_info (context, TRUE);
 
+  target = cairo_get_target (cr);
+
+  if (!info->surface_options)
+    info->surface_options = cairo_font_options_create ();
+  cairo_surface_get_font_options (target, info->surface_options);
+
+  old_merged_options = info->merged_options;
+  info->merged_options = NULL;
+
+  merged_options = _pango_cairo_context_get_merged_font_options (context);
+
+  if (old_merged_options)
+    {
+      if (!cairo_font_options_equal (merged_options, old_merged_options))
+	changed = TRUE;
+      cairo_font_options_destroy (old_merged_options);
+      old_merged_options = NULL;
+    }
+  else
+    changed = TRUE;
+
   cairo_get_matrix (cr, &cairo_matrix);
   pango_matrix.xx = cairo_matrix.xx;
   pango_matrix.yx = cairo_matrix.yx;
   pango_matrix.xy = cairo_matrix.xy;
   pango_matrix.yy = cairo_matrix.yy;
-  pango_matrix.x0 = cairo_matrix.x0;
-  pango_matrix.y0 = cairo_matrix.y0;
+  pango_matrix.x0 = 0;
+  pango_matrix.y0 = 0;
 
-  layout_matrix = pango_context_get_matrix (context);
-  if (!layout_matrix)
-    layout_matrix = &identity_matrix;
+  current_matrix = pango_context_get_matrix (context);
+  if (!current_matrix)
+    current_matrix = &identity_matrix;
 
-  if (0 != memcmp (&pango_matrix, layout_matrix, sizeof (pango_matrix)))
+  /* layout is matrix-independent if metrics-hinting is off.
+   * also ignore matrix translation offsets */
+  if ((cairo_font_options_get_hint_metrics (merged_options) != CAIRO_HINT_METRICS_OFF) &&
+      (0 != memcmp (&pango_matrix, current_matrix, sizeof (PangoMatrix))))
     changed = TRUE;
 
   pango_context_set_matrix (context, &pango_matrix);
 
 
-  target = cairo_get_target (cr);
-
-  if (!info->surface_options) {
-    info->surface_options = cairo_font_options_create ();
-    changed = TRUE;
-    cairo_surface_get_font_options (target, info->surface_options);
-  } else {
-    cairo_font_options_t *surface_options = cairo_font_options_create ();
-    cairo_surface_get_font_options (target, surface_options);
-    if (!cairo_font_options_equal (surface_options, info->surface_options))
-      {
-	cairo_surface_get_font_options (target, info->surface_options);
-	changed = TRUE;
-      }
-    cairo_font_options_destroy (surface_options);
-  }
-
-  if (info->merged_options)
-    {
-      cairo_font_options_destroy (info->merged_options);
-      info->merged_options = NULL;
-    }
-
   return changed;
 }
 
 /**
  * pango_cairo_update_context:
  * @cr: a Cairo context
- * @context: a #PangoContext, from pango_cairo_font_map_create_context()
+ * @context: a #PangoContext, from a pangocairo font map
  *
  * Updates a #PangoContext previously created for use with Cairo to
  * match the current transformation and target surface of a Cairo
@@ -164,7 +167,7 @@ pango_cairo_update_context (cairo_t      *cr,
 
 /**
  * pango_cairo_context_set_resolution:
- * @context: a #PangoContext, from pango_cairo_font_map_create_context()
+ * @context: a #PangoContext, from a pangocairo font map
  * @dpi: the resolution in "dots per inch". (Physical inches aren't actually
  *   involved; the terminology is conventional.) A 0 or negative value
  *   means to use the resolution from the font map.
@@ -186,7 +189,7 @@ pango_cairo_context_set_resolution (PangoContext *context,
 
 /**
  * pango_cairo_context_get_resolution:
- * @context: a #PangoContext, from pango_cairo_font_map_create_context()
+ * @context: a #PangoContext, from a pangocairo font map
  *
  * Gets the resolution for the context. See pango_cairo_context_set_resolution()
  *
@@ -208,7 +211,7 @@ pango_cairo_context_get_resolution (PangoContext *context)
 
 /**
  * pango_cairo_context_set_font_options:
- * @context: a #PangoContext, from pango_cairo_font_map_create_context()
+ * @context: a #PangoContext, from a pangocairo font map
  * @options: a #cairo_font_options_t, or %NULL to unset any previously set
  *           options. A copy is made.
  *
@@ -245,10 +248,10 @@ pango_cairo_context_set_font_options (PangoContext               *context,
 
 /**
  * pango_cairo_context_get_font_options:
- * @context: a #PangoContext, from pango_cairo_font_map_create_context()
+ * @context: a #PangoContext, from a pangocairo font map
  *
  * Retrieves any font rendering options previously set with
- * pango_cairo_font_map_set_font_options(). This functions not report options
+ * pango_cairo_font_map_set_font_options(). This function does not report options
  * that are derived from the target surface by pango_cairo_update_context()
  *
  * Return value: the font options previously set on the context, or %NULL
@@ -257,7 +260,7 @@ pango_cairo_context_set_font_options (PangoContext               *context,
  *
  * Since: 1.10
  **/
-G_CONST_RETURN cairo_font_options_t *
+const cairo_font_options_t *
 pango_cairo_context_get_font_options (PangoContext *context)
 {
   PangoCairoContextInfo *info;
@@ -283,7 +286,7 @@ pango_cairo_context_get_font_options (PangoContext *context)
  * Return value: the combined set of font options. This value is owned
  * by the context and must not be modified or freed.
  **/
-G_CONST_RETURN cairo_font_options_t *
+const cairo_font_options_t *
 _pango_cairo_context_get_merged_font_options (PangoContext *context)
 {
   PangoCairoContextInfo *info = get_context_info (context, TRUE);
@@ -303,7 +306,7 @@ _pango_cairo_context_get_merged_font_options (PangoContext *context)
 
 /**
  * pango_cairo_context_set_shape_renderer:
- * @context: a #PangoContext, from pango_cairo_font_map_create_context()
+ * @context: a #PangoContext, from a pangocairo font map
  * @func: Callback function for rendering attributes of type
  * %PANGO_ATTR_SHAPE, or %NULL to disable shape rendering.
  * @data: User data that will be passed to @func.
@@ -337,8 +340,8 @@ pango_cairo_context_set_shape_renderer (PangoContext                *context,
 }
 
 /**
- * pango_cairo_context_get_shape_renderer:
- * @context: a #PangoContext, from pango_cairo_font_map_create_context()
+ * pango_cairo_context_get_shape_renderer: (skip)
+ * @context: a #PangoContext, from a pangocairo font map
  * @data: Pointer to #gpointer to return user data
  *
  * Sets callback function for context to use for rendering attributes
@@ -378,6 +381,39 @@ pango_cairo_context_get_shape_renderer (PangoContext                *context,
     }
 }
 
+/**
+ * pango_cairo_create_context:
+ * @cr: a Cairo context
+ *
+ * Creates a context object set up to match the current transformation
+ * and target surface of the Cairo context.  This context can then be
+ * used to create a layout using pango_layout_new().
+ *
+ * This function is a convenience function that creates a context using
+ * the default font map, then updates it to @cr.  If you just need to
+ * create a layout for use with @cr and do not need to access #PangoContext
+ * directly, you can use pango_cairo_create_layout() instead.
+ *
+ * Return value: (transfer full): the newly created #PangoContext. Free with
+ *   g_object_unref().
+ *
+ * Since: 1.22
+ **/
+PangoContext *
+pango_cairo_create_context (cairo_t *cr)
+{
+  PangoFontMap *fontmap;
+  PangoContext *context;
+
+  g_return_val_if_fail (cr != NULL, NULL);
+
+  fontmap = pango_cairo_font_map_get_default ();
+  context = pango_font_map_create_context (fontmap);
+  pango_cairo_update_context (cr, context);
+
+  return context;
+}
+
 /**
  * pango_cairo_create_layout:
  * @cr: a Cairo context
@@ -394,7 +430,7 @@ pango_cairo_context_get_shape_renderer (PangoContext                *context,
  * #PangoContext object for each layout. This might matter in an
  * application that was laying out large amounts of text.
  *
- * Return value: the newly created #PangoLayout. Free with
+ * Return value: (transfer full): the newly created #PangoLayout. Free with
  *   g_object_unref().
  *
  * Since: 1.10
@@ -402,17 +438,13 @@ pango_cairo_context_get_shape_renderer (PangoContext                *context,
 PangoLayout *
 pango_cairo_create_layout  (cairo_t *cr)
 {
-  PangoFontMap *fontmap;
   PangoContext *context;
   PangoLayout *layout;
 
   g_return_val_if_fail (cr != NULL, NULL);
 
-  fontmap = pango_cairo_font_map_get_default ();
-  context = pango_cairo_font_map_create_context ((PangoCairoFontMap *) (fontmap));
+  context = pango_cairo_create_context (cr);
   layout = pango_layout_new (context);
-
-  pango_cairo_update_context (cr, context);
   g_object_unref (context);
 
   return layout;
